unit IconImage;

interface

uses
  Windows, SysUtils, Classes, Graphics, Icontypes, IconTools;

type
  TIconImage = class
  private
  protected
    fHandle: HIcon;
    fInfo: TIconResInfo;
    fIconData: Pointer;
    function GetHandle: HIcon; virtual;
    function GetIcon: TIcon; virtual;
    procedure CorrectIconSize;
  public
    constructor Create;
    constructor CreateCopy(Source: TIconImage); virtual;
    destructor Destroy; override;
    procedure Draw(Canvas: TCanvas; X,Y: integer);
    procedure WriteIconDataToStream(Stream: TStream);
    property Handle: HIcon read GetHandle;
    property Icon: TIcon read GetIcon;
    property Info: TIconResInfo read FInfo;
  end;

  TFileIconImage = class(TIconImage)
  public
    constructor Create(Stream: TStream);
  end;

  TResourceIconImage = class(TIconImage)
  public
    constructor Create(GroupIconStream: TStream; Instance: THandle);
  end;

implementation

constructor TResourceIconImage.Create(GroupIconStream : TStream;Instance : Thandle);
var
  Stream: TResourceStream;
  ResInfo: TResourceIconResInfo;
begin
  inherited Create;
  GroupIconStream.Read(ResInfo,SizeOf(ResInfo));
  fInfo:=ResInfo.ResInfo;
  Stream:=TResourceStream.CreateFromID(Instance,ResInfo.ID,RT_Icon);
  try
    GetMem(fIconData,Stream.Size);
    Stream.Read(fIcondata^,Stream.Size)
  finally
    Stream.free
  end;
  CorrectIconSize
end;

constructor TFileIconImage.Create(Stream: TStream);
var
  ResInfo: TFileIconResInfo;
  OldPos: Cardinal;
begin
  inherited Create;
  Stream.Read(ResInfo,SizeOf(ResInfo));
  OldPos:=Stream.Position;
  fInfo:=ResInfo.ResInfo;
  Stream.Position:=ResInfo.dwImageOffset;
  try
    GetMem(fIconData,ResInfo.ResInfo.BytesInRes);
    Stream.Read(fIconData^,ResInfo.ResInfo.BytesInRes)
  finally
    Stream.Position:=OldPos
  end;
  CorrectIconSize
end;
 
constructor TIconImage.Create;
begin
  fHandle:=0; fIconData:=nil
end;

constructor TIconImage.CreateCopy(Source: TIconImage);
begin
  Create;
  fInfo:=Source.Info;
  Getmem(fIcondata,fInfo.BytesInRes);
  if Assigned(Source.fIconData) then
    System.Move(Source.fIconData^,fIconData^,fInfo.BytesInRes);
  if (Source.Handle<>0) then fHandle:=CopyIcon(Source.Handle);
  CorrectIconSize
end;

procedure TIconImage.CorrectIconSize;
var
  X,Y: integer;
  BInfo: PBitMapInfoHeader;
begin
  BInfo:=PBitmapInfoHeader(fIconData);
  X:=BInfo^.biWidth; Y:=BInfo^.biHeight shr 1;
  if (X<Y) then
    Y:=X
  else
    X:=Y;
  fInfo.Width:=X; fInfo.Height:=Y
end;

function TIconImage.GetHandle: HIcon;
begin
  if (fHandle=0) then
    begin
      with Info do
        if Assigned(fIconData) then
          fHandle:=CreateIconFromResourceEx(fIconData,BytesInRes,True,Win3,Width,Height,LR_DEFAULTCOLOR)
    end;
  Result:=fHandle
end;

procedure TIconImage.WriteIconDataToStream(Stream: TStream);
begin
  if not Assigned(fIcondata) then exit;
  Stream.Write(fIconData^,Info.BytesInRes)
end;

function TIconImage.GetIcon: TIcon;
begin
  Result:=THiResIcon.Create;
  Result.Handle:=GetHandle
end;

procedure TIconImage.Draw(Canvas: TCanvas; X,Y: integer);
begin
  DrawIconEx(Canvas.Handle,X,Y,Handle,Info.Height,Info.Width,0,0,DI_NORMAL)
end;

destructor TIconImage.Destroy;
begin
  DestroyIcon(fHandle);
  if Assigned(fIconData) then
    Freemem(fIconData,Info.BytesInRes)
end;

end.
