unit IconTools;

interface

uses
  SysUtils, Windows, Graphics;

type
  THiResIcon = class(TIcon)
  private
    fHeight: integer;
    fWidth: integer;
    fIconChanged: boolean;
  protected
    procedure GetIconSizeInfo; virtual;
    procedure Changed(Sender: TObject); override;
    procedure Draw(ACanvas: TCanvas; const Rect: TRect); override;
    function GetHeight: integer; override;
    function GetWidth: integer; override;
  public
    constructor Create; override;
  end;

implementation

constructor THiresIcon.Create;
begin
  inherited;
  fHeight:=GetSystemMetrics(SM_CYICON);
  fWidth:=GetSystemMetrics(SM_CXICON);
  fIconChanged:=False
end;

procedure THiResIcon.Changed(Sender: TObject);
begin
  inherited;
  fIconChanged:=True
end;

procedure THiResIcon.Draw(ACanvas: TCanvas; const Rect: TRect);
var
  DC: HDC;
begin
  with Rect.TopLeft do
    begin
      GetIconSizeInfo;
      DC:=ACanvas.Handle;
      if DC <> 0 then
        DrawIconEx(DC,X,Y,Handle,Width,Height,0,0,DI_NORMAL)
    end
end;

procedure THiResIcon.GetIconSizeInfo;
Var
  IconInfo: TIconInfo;
  Bitmap: Windows.TBitmap;
  Res: integer;
begin
  if not fIconChanged then exit;
  fIconChanged:=False;
  if not GetIconInfo(Handle,IconInfo) then exit;
  Res:=GetObject(IconInfo.hbmColor,SizeOf(Bitmap),@Bitmap);
  if Res = 0 then Res:=GetObject(IconInfo.hbmMask,SizeOf(Bitmap),@Bitmap);
  if Res = 0 then exit;
  DeleteObject(IconInfo.hbmColor);
  DeleteObject(IconInfo.hbmMask);
  fWidth:=Bitmap.bmWidth; fHeight:=Bitmap.bmWidth
end;

function THiResIcon.GetHeight: integer;
begin
  GetIconSizeInfo;
  Result:=fHeight
end;

function THiResIcon.GetWidth: integer;
begin
  GetIconSizeInfo;
  Result:=fWidth
end;

end.
